#include "course.hpp"
#define IMGUI_DEFINE_MATH_OPERATORS
#include "imgui.h"
#include "imgui/misc/cpp/imgui_stdlib.h"
#include <optional>
#include <unordered_map>

Course::Course(std::string in) {
    number = in;
}

CourseWindow::CourseWindow(Course in_course) : course(in_course) {}

void CourseWindow::Show() {
    ImGui::SetNextWindowSize(ImVec2(256, 275), ImGuiCond_FirstUseEver);
    if (ImGui::Begin(course.number.c_str(), &Open)) {
        link_pressed = false;
        ImGui::Text("%s", course.name.c_str());
        ImGui::TextWrapped("%s", course.desc.c_str());
        ImGui::SeparatorText("Prereqs:");
        for (int i = 0; i < course.prereq.size(); i++) {
            if (i > 0) {
                ImGui::SeparatorText("and");
            }
            for (int j = 0; j < course.prereq[i].size(); j++) {
                bool temppressed = ImGui::Button(course.prereq[i][j].c_str());
                if (temppressed) {
                    new_course_string = course.prereq[i][j].c_str();
                    link_pressed = true;
                }
                if (course.prereq[i].size() > 1 && j < (course.prereq[i].size() - 1)) {
                    ImGui::Text("Or");
                }
            }
        }
        if (course.prereq.size() == 0) {
            ImGui::Text("None");
        }
    }
    window_pos = ImGui::GetWindowPos();
    window_size = ImGui::GetWindowSize();
    ImGui::End();
}

CourseManager::CourseManager(std::unordered_map<std::string, Course> ac) : all_courses(ac) {}

void CourseManager::Draw() {

    for (auto it = course_windows.begin(); it != course_windows.end(); ++it) {
        if (it->second.Open) {
            it->second.Show();
            for (int i = 0; i < it->second.course.prereq.size(); i++) {
                for (int j = 0; j < it->second.course.prereq[i].size(); j++) {
                    if (course_windows.count(it->second.course.prereq[i][j])) {
                        auto target = course_windows.find(it->second.course.prereq[i][j]);
                        auto target_ref = target->second;
                        auto us = it->second;
                        ImVec2 center_us = ImVec2((us.window_pos.x + us.window_size.x * 0.5f),
                                                  (us.window_pos.y + us.window_size.y * 0.5f));
                        ImVec2 center_target =
                            ImVec2((target_ref.window_pos.x + target_ref.window_size.x * 0.5f),
                                   (target_ref.window_pos.y + target_ref.window_size.y * 0.5f));
                        if (target->second.Open) {
                            ImGui::GetBackgroundDrawList()->AddLine(center_us, center_target,
                                                                    IM_COL32_WHITE, 5);
                        }
                    }
                }
            }
        }
        if (it->second.link_pressed) {
            AddByNum(it->second.new_course_string);
        }
    }
}

void CourseManager::AddByNum(std::string num) {
    if (course_windows.count(num) > 0) {
        auto course_it = course_windows.find(num);
        course_it->second.Open = true;
    }
    auto course_it = all_courses.find(num);
    if (course_it != all_courses.end()) {
        Course c = course_it->second;

        course_windows.insert(std::make_pair(num, CourseWindow(c)));
    }
}
